@@ -46,6 +46,8 @@ from nion.ui import UserInterface
 from nion.ui import Window as UIWindow
 from nion.utils import Event
 from nion.utils import Geometry
+from nion.utils import ListModel
+from nion.utils import Model
 from nion.utils import Registry
 
 _ = gettext.gettext
@@ -74,7 +76,7 @@ class Application(UIApplication.BaseApplication):
 
         self.ui.persistence_root = "3"  # sets of preferences
         self.__resources_path = resources_path
-        self.version_str = "0.15.0"
+        self.version_str = "0.15.2"
 
         self.document_model_available_event = Event.Event()
 
@@ -123,15 +125,14 @@ class Application(UIApplication.BaseApplication):
 
     def deinitialize(self):
         # shut down hardware source manager, unload plug-ins, and really exit ui
+        if self.__profile:
+            self.__profile.close()
+            self.__profile = None
         HardwareSource.HardwareSourceManager().close()
         PlugInManager.unload_plug_ins()
         self.__class__.count -= 1
         super().deinitialize()
 
-    def run(self):
-        """Alternate start which allows ui to control event loop."""
-        self.ui.run(self)
-
     @property
     def profile(self) -> typing.Optional[Profile.Profile]:
         return self.__profile
@@ -174,17 +175,66 @@ class Application(UIApplication.BaseApplication):
             profile, is_created = self.__establish_profile(profile_path)
         self.__profile = profile
 
-        # if it was created, it probably means it is migrating from an old version. so add all recent projects.
-        # they will initially be disabled and the user will have to explicitly upgrade them.
-        if is_created:
-            for library_path in self.get_recent_library_paths():
-                logging.getLogger("loader").info(f"Adding legacy project {library_path}")
-                profile.add_project_folder(pathlib.Path(library_path), load=False)
-
         # configure the document model object.
         DocumentModel.DocumentModel.computation_min_period = 0.1
         DocumentModel.DocumentModel.computation_min_factor = 1.0
 
+        # if it was created, it probably means it is migrating from an old version. so add all recent projects.
+        # they will initially be disabled and the user will have to explicitly upgrade them.
+        if is_created:
+            # present a dialog for showing progress while finding existing projects
+            u = Declarative.DeclarativeUI()
+            task_message = u.create_label(text=_("Looking for existing projects..."))
+            progress = u.create_progress_bar(value="@binding(progress_value_model.value)", width=300 - 24)
+            progress_message = u.create_label(text="@binding(message_str_model.value)")
+            main_column = u.create_column(task_message, progress, progress_message, spacing=8, width=300)
+            window = u.create_window(main_column, title=_("Locating Existing Projects"), margin=12, window_style="tool")
+
+            # handler for the progress window. defines two value models: progress, an int 0-100; and message, a string.
+            class FindExistingProjectsWindowHandler(Declarative.WindowHandler):
+                def __init__(self, *, completion_fn: typing.Optional[typing.Callable[[], None]] = None):
+                    super().__init__(completion_fn=completion_fn)
+                    self.progress_value_model = Model.PropertyModel(0)
+                    self.message_str_model = Model.PropertyModel(str())
+
+            # construct the window handler and run it. when the dialog closes, it will continue by
+            # calling open default project.
+            window_handler = FindExistingProjectsWindowHandler(completion_fn=functools.partial(self.__open_default_project, profile_dir, is_created))
+            window_handler.run(window, app=self)
+
+            # define an async routine that will perform the finding of the existing projects.
+            # this is just a loop that yields via asyncio.sleep periodically. the loop loads
+            # the projects and updates the progress and message value models in the dialog.
+            # when finished, it asks the window to close on its next periodic call. it is
+            # necessary to close this way because the close request may close the event loop
+            # in which we're executing. so queueing the close request avoids that.
+            async def find_existing_projects():
+                recent_library_paths = self.get_recent_library_paths()
+                for index, library_path in enumerate(recent_library_paths):
+                    window_handler.progress_value_model.value = 100 * index // len(recent_library_paths)
+                    window_handler.message_str_model.value = str(library_path.name)
+                    logging.getLogger("loader").info(f"Adding existing project {index + 1}/{len(recent_library_paths)} {library_path}")
+                    await asyncio.sleep(0)
+                    profile.add_project_folder(pathlib.Path(library_path), load=False)
+                window_handler.progress_value_model.value = 100
+                window_handler.message_str_model.value = _("Finished")
+                await asyncio.sleep(1)
+                window_handler.window.queue_request_close()
+
+            # launch the find existing projects task asynchronously.
+            window_handler.window.event_loop.create_task(find_existing_projects())
+            return True
+        else:
+            # continue with opening the default project
+            return self.__open_default_project(profile_dir, is_created)
+
+    def __open_default_project(self, profile_dir: pathlib.Path, is_created: bool) -> bool:
+        # if the default project is known, open it.
+        # if it fails, ask the user to select another project.
+        # if no default project is known, ask the user to choose one.
+
+        profile = self.__profile
+
         project_reference: typing.Optional[Profile.ProjectReference] = None
 
         project_reference = project_reference or profile.get_project_reference(profile.last_project_reference)
@@ -237,81 +287,160 @@ class Application(UIApplication.BaseApplication):
         return document_controller
 
     def show_open_project_dialog(self) -> None:
-        ui = self.ui
-        filter = "Projects (*.nsproj);;Legacy Libraries (*.nslib);;All Files (*.*)"
-        import_dir = ui.get_persistent_string("open_directory", ui.get_document_location())
-        paths, selected_filter, selected_directory = ui.get_file_paths_dialog(_("Add Existing Library"), import_dir, filter)
-        ui.set_persistent_string("open_directory", selected_directory)
-        if len(paths) == 1:
-            project_reference = self.profile.open_project(pathlib.Path(paths[0]))
-            if project_reference:
-                self.open_project_reference(project_reference)
-            else:
-                self.show_ok_dialog(_("Error Opening Project"), _("Unable to open project."), completion_fn=self.show_choose_project_dialog)
+        with self.prevent_close():
+            ui = self.ui
+            filter = "Projects (*.nsproj);;Legacy Libraries (*.nslib);;All Files (*.*)"
+            import_dir = ui.get_persistent_string("open_directory", ui.get_document_location())
+            paths, selected_filter, selected_directory = ui.get_file_paths_dialog(_("Add Existing Library"), import_dir, filter)
+            ui.set_persistent_string("open_directory", selected_directory)
+            if len(paths) == 1:
+                project_reference = self.profile.open_project(pathlib.Path(paths[0]))
+                if project_reference:
+                    self.open_project_reference(project_reference)
+                else:
+                    self.show_ok_dialog(_("Error Opening Project"), _("Unable to open project."), completion_fn=self.show_choose_project_dialog)
 
     def show_choose_project_dialog(self) -> None:
-        u = Declarative.DeclarativeUI()
-        button_row = u.create_row(u.create_push_button(text=_("New..."), on_clicked="new_project"),
-                                  u.create_push_button(text=_("Open..."), on_clicked="open_project"),
-                                  u.create_stretch(),
-                                  u.create_push_button(text=_("Cancel"), on_clicked="close_window"),
-                                  u.create_push_button(text=_("Open Recent")),
-                                  spacing=8)
+        with self.prevent_close():
+            u = Declarative.DeclarativeUI()
+            button_row = u.create_row(u.create_push_button(text=_("New..."), on_clicked="new_project"),
+                                      u.create_push_button(text=_("Open..."), on_clicked="open_project"),
+                                      u.create_stretch(),
+                                      u.create_push_button(text=_("Cancel"), on_clicked="close_window"),
+                                      u.create_push_button(text=_("Open Recent"), on_clicked="open_recent"),
+                                      spacing=8)
 
-        used_project_references = list()
-        project_titles = list()
+            project_references_model = ListModel.FilteredListModel(container=self.__profile, items_key="project_references")
+            project_references_model.filter = ListModel.PredicateFilter(lambda pr: pr.project_state != "loaded")
+            project_references_model.sort_key = lambda pr: pr.last_used
+            project_references_model.sort_reverse = True
 
-        project_references = filter(lambda pr: pr.project_state != "loaded", self.__profile.project_references)
-        project_references = sorted(project_references, key=operator.attrgetter("last_used"), reverse=True)
-        for project_reference in project_references[:20]:
-            if project_reference.project_state != "loaded":
-                project_title = project_reference.title
-                if project_reference.project_state == "needs_upgrade":
-                    project_title += " " + _("(NEEDS UPGRADE)")
-                elif project_reference.project_state != "unloaded" or project_reference.project_version != FileStorageSystem.PROJECT_VERSION:
-                    project_title += " " + _("(MISSING OR UNREADABLE)")
-                used_project_references.append(project_reference)
-                project_titles.append(project_title)
+            class ProjectReferenceItem:
+                # provides a str converter and a tool tip.
+                def __init__(self, project_reference: Profile.ProjectReference):
+                    self.project_reference = project_reference
 
-        item_list = u.create_list_box(items=project_titles, height=240, min_height=180, size_policy_horizontal="expanding", on_item_selected="recent_item_selected")
+                def __str__(self) -> str:
+                    project_reference = self.project_reference
+                    project_title = project_reference.title
+                    if project_reference.project_state == "needs_upgrade":
+                        project_title += " " + _("(NEEDS UPGRADE)")
+                    elif project_reference.project_state != "unloaded" or project_reference.project_version != FileStorageSystem.PROJECT_VERSION:
+                        project_title += " " + _("(MISSING OR UNREADABLE)")
+                    return project_title
 
-        main_column = u.create_column(u.create_label(text=_("Recent Projects")),
-                                      item_list,
-                                      u.create_spacing(13),
-                                      button_row, spacing=8, width=380)
-        window = u.create_window(main_column, title=_("Choose Project"), margin=12, window_style="tool")
+                @property
+                def tool_tip(self) -> str:
+                    return str(self.project_reference.path)
 
-        def open_project_reference(project_reference: Profile.ProjectReference) -> None:
-            self.open_project_reference(project_reference)
+            project_reference_items_model = ListModel.MappedListModel(container=project_references_model,
+                                                                      master_items_key="project_references",
+                                                                      items_key="project_reference_items",
+                                                                      map_fn=ProjectReferenceItem)
 
-        def show_open_project_dialog() -> None:
-            self.show_open_project_dialog()
+            item_list = u.create_list_box(items_ref="@binding(list_property_model.value)",
+                                          current_index="@binding(current_index)",
+                                          height=240, min_height=180, size_policy_horizontal="expanding",
+                                          on_item_selected="recent_item_selected",
+                                          on_item_handle_context_menu="item_handle_context_menu")
 
-        def show_new_project_dialog() -> None:
-            NewProjectAction().invoke(UIWindow.ActionContext(self, None, None))
+            main_column = u.create_column(u.create_label(text=_("Recent Projects")),
+                                          item_list,
+                                          u.create_spacing(13),
+                                          button_row, spacing=8, width=380)
+            window = u.create_window(main_column, title=_("Choose Project"), margin=12, window_style="tool")
 
-        class ChooseProjectHandler(Declarative.WindowHandler):
-            def __init__(self):
-                super().__init__()
+            def open_project_reference(project_reference: Profile.ProjectReference) -> None:
+                self.open_project_reference(project_reference)
 
-            def recent_item_selected(self, widget: Declarative.UIWidget, current_index: int) -> None:
-                if 0 <= current_index < len(used_project_references):
-                    self.close_window()
-                    open_project_reference(used_project_references[current_index])
+            def show_open_project_dialog() -> None:
+                self.show_open_project_dialog()
 
-            def new_project(self, widget: Declarative.UIWidget) -> None:
-                self.close_window()
-                show_new_project_dialog()
+            def show_new_project_dialog() -> None:
+                NewProjectAction().invoke(UIWindow.ActionContext(self, None, None))
 
-            def open_project(self, widget: Declarative.UIWidget) -> None:
-                self.close_window()
-                show_open_project_dialog()
+            from nion.utils import Observable
 
-        ChooseProjectHandler().run(window, app=self)
+            class ListPropertyModel(Observable.Observable):
+                # copied from nionutils to avoid requiring new version.
+                # remove this code once nionutils 0.3.24+ is released.
 
-    def get_recent_library_paths(self):
+                def __init__(self, list_model):
+                    super().__init__()
+                    self.__list_model = list_model
+                    self.__item_inserted_event_listener = list_model.item_inserted_event.listen(self.__item_inserted)
+                    self.__item_removed_event_listener = list_model.item_removed_event.listen(self.__item_removed)
+
+                def close(self) -> None:
+                    self.__list_model = None
+                    self.__item_inserted_event_listener = None
+                    self.__item_removed_event_listener = None
+
+                def __item_inserted(self, key: str, item, before_index: int) -> None:
+                    self.notify_property_changed("value")
+
+                def __item_removed(self, key: str, item, index: int) -> None:
+                    self.notify_property_changed("value")
+
+                @property
+                def value(self):
+                    return self.__list_model.items
+
+            class ChooseProjectHandler(Declarative.WindowHandler):
+                def __init__(self, application: Application):
+                    super().__init__()
+                    self.__application = application
+                    self.current_index = 0
+                    self.list_property_model = ListPropertyModel(project_reference_items_model)
+
+                def recent_item_selected(self, widget: Declarative.UIWidget, current_index: int) -> None:
+                    if 0 <= current_index < len(project_reference_items_model.project_reference_items):
+                        # to ensure the application does not close upon closing the last window, force it
+                        # to stay open while the window is closed and another reopened.
+                        with self.__application.prevent_close():
+                            self.close_window()
+                            project_reference_item = project_reference_items_model.project_reference_items[current_index]
+                            open_project_reference(project_reference_item.project_reference)
+
+                def new_project(self, widget: Declarative.UIWidget) -> None:
+                    # to ensure the application does not close upon closing the last window, force it
+                    # to stay open while the window is closed and another reopened.
+                    with self.__application.prevent_close():
+                        show_new_project_dialog()
+                        self.close_window()
+
+                def open_project(self, widget: Declarative.UIWidget) -> None:
+                    # to ensure the application does not close upon closing the last window, force it
+                    # to stay open while the window is closed and another reopened.
+                    with self.__application.prevent_close():
+                        show_open_project_dialog()
+                        self.close_window()
+
+                def open_recent(self, widget: Declarative.UIWidget) -> None:
+                    self.recent_item_selected(widget, self.current_index)
+
+                def item_handle_context_menu(self, widget: Declarative.UIWidget, *,
+                                             gx: int, gy: int,
+                                             index: typing.Optional[int], **kwargs) -> bool:
+                    if index is not None:
+                        project_reference_item = project_reference_items_model.project_reference_items[index]
+                        menu = self.window.create_context_menu()
+                        menu.add_menu_item(_(f"Open Project Location"), functools.partial(ProjectPanel.reveal_project, project_reference_item.project_reference))
+                        menu.add_separator()
+
+                        def remove_project(index: int) -> None:
+                            project_reference_item = project_reference_items_model.project_reference_items[index]
+                            self.__application.profile.remove_project_reference(project_reference_item.project_reference)
+
+                        menu.add_menu_item(_(f"Remove Project from List"), functools.partial(remove_project, index))
+                        menu.popup(gx, gy)
+                    return True
+
+            ChooseProjectHandler(self).run(window, app=self)
+
+    def get_recent_library_paths(self) -> typing.List[pathlib.Path]:
         workspace_history = self.ui.get_persistent_object("workspace_history", list())
-        return [file_path for file_path in workspace_history if os.path.exists(file_path)]
+        return [pathlib.Path(file_path) for file_path in workspace_history if os.path.exists(file_path)]
 
     def create_document_controller(self, document_model, workspace_id, display_item=None):
         self._set_document_model(document_model)  # required to allow API to find document model
@@ -326,7 +455,7 @@ class Application(UIApplication.BaseApplication):
         document_controller.show()
         return document_controller
 
-    def _set_profile_for_test(self, profile: Profile.Profile) -> None:
+    def _set_profile_for_test(self, profile: typing.Optional[Profile.Profile]) -> None:
         self.__profile = profile
 
     def __establish_profile(self, profile_path: pathlib.Path) -> typing.Tuple[typing.Optional[Profile.Profile], bool]:
@@ -397,25 +526,26 @@ class Application(UIApplication.BaseApplication):
             self.switch_project_reference(project_reference)
 
     def open_project_reference(self, project_reference: Profile.ProjectReference) -> None:
-        if project_reference.project_version == FileStorageSystem.PROJECT_VERSION and project_reference.project_state == "unloaded":
-            self.switch_project_reference(project_reference)
-        elif project_reference.project_state == "needs_upgrade":
-            def handle_upgrade(result: bool) -> None:
-                if result:
-                    try:
-                        new_project_reference = self.profile.upgrade(project_reference)
-                    except Exception:
-                        self.show_ok_dialog(_("Error Upgrading Project"), _("Unable to upgrade project."))
-                        new_project_reference = None
-                    if new_project_reference:
-                        self.switch_project_reference(new_project_reference)
+        with self.prevent_close():
+            if project_reference.project_version == FileStorageSystem.PROJECT_VERSION and project_reference.project_state == "unloaded":
+                self.switch_project_reference(project_reference)
+            elif project_reference.project_state == "needs_upgrade":
+                def handle_upgrade(result: bool) -> None:
+                    if result:
+                        try:
+                            new_project_reference = self.profile.upgrade(project_reference)
+                        except Exception:
+                            self.show_ok_dialog(_("Error Upgrading Project"), _("Unable to upgrade project."))
+                            new_project_reference = None
+                        if new_project_reference:
+                            self.switch_project_reference(new_project_reference)
 
-            self.show_ok_cancel_dialog(_("Project Needs Upgrade"),
-                                       _("This project needs to be upgraded to work with this version."),
-                                       ok_text=_("Upgrade"),
-                                       completion_fn=handle_upgrade)
-        else:
-            self.show_ok_dialog(_("Error Opening Project"), _("Unable to open project."), completion_fn=self.show_choose_project_dialog)
+                self.show_ok_cancel_dialog(_("Project Needs Upgrade"),
+                                           _("This project needs to be upgraded to work with this version."),
+                                           ok_text=_("Upgrade"),
+                                           completion_fn=handle_upgrade)
+            else:
+                self.show_ok_dialog(_("Error Opening Project"), _("Unable to open project."), completion_fn=self.show_choose_project_dialog)
 
     def switch_project_reference(self, project_reference: Profile.ProjectReference) -> None:
         for window in self.windows:
