@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 # standard libraries
-import copy
+import collections
 import functools
 import gettext
 import operator
@@ -9,7 +9,6 @@ import random
 import string
 import sys
 import typing
-import uuid
 import weakref
 
 # third party libraries
@@ -24,6 +23,7 @@ from nion.swift.model import DataItem
 from nion.swift.model import DataStructure
 from nion.swift.model import DisplayItem
 from nion.swift.model import Graphics
+from nion.swift.model import Persistence
 from nion.swift.model import Schema
 from nion.swift.model import Symbolic
 from nion.ui import CanvasItem
@@ -48,7 +48,9 @@ _ = gettext.gettext
 
 class AddVariableCommand(Undo.UndoableCommand):
 
-    def __init__(self, document_model, computation: Symbolic.Computation, name: str=None, value_type: str=None, value=None, value_default=None, value_min=None, value_max=None, control_type: str=None, specifier: dict=None, label: str=None):
+    def __init__(self, document_model, computation: Symbolic.Computation, name: str = None, value_type: str = None,
+                 value=None, value_default=None, value_min=None, value_max=None, control_type: str = None,
+                 specified_item: typing.Optional[Persistence.PersistentObject] = None, label: str = None):
         super().__init__(_("Add Computation Variable"))
         self.__document_model = document_model
         self.__computation_proxy = computation.create_proxy()
@@ -59,7 +61,7 @@ class AddVariableCommand(Undo.UndoableCommand):
         self.__value_min = value_min
         self.__value_max = value_max
         self.__control_type = control_type
-        self.__specifier = specifier
+        self.__specified_item = specified_item
         self.__label = label
         self.__variable_index = None
         self.initialize()
@@ -74,14 +76,14 @@ class AddVariableCommand(Undo.UndoableCommand):
         self.__value_min = None
         self.__value_max = None
         self.__control_type = None
-        self.__specifier = None
+        self.__specified_item = None
         self.__label = None
         self.__variable = None
         super().close()
 
     def perform(self):
-        computation = self.__computation_proxy.item
-        variable = computation.create_variable(self.__name, self.__value_type, self.__value, self.__value_default, self.__value_min, self.__value_max, self.__control_type, self.__specifier, self.__label)
+        computation = typing.cast(Symbolic.Computation, self.__computation_proxy.item)
+        variable = computation.create_variable(self.__name, self.__value_type, self.__value, self.__value_default, self.__value_min, self.__value_max, self.__control_type, self.__specified_item, self.__label)
         self.__variable_index = computation.variables.index(variable)
 
     def _get_modified_state(self):
@@ -231,57 +233,6 @@ class ChangeComputationCommand(Undo.UndoableCommand):
         return isinstance(command, ChangeComputationCommand) and self.command_id and self.command_id == command.command_id and self.__computation_proxy.item == command.__computation_proxy.item
 
 
-class ChangeVariableCommand(Undo.UndoableCommand):
-
-    def __init__(self, document_model, computation: Symbolic.Computation, variable: Symbolic.ComputationVariable, *, title: str=None, command_id: str=None, is_mergeable: bool=False, **kwargs):
-        super().__init__(title if title else _("Change Computation Variable"), command_id=command_id, is_mergeable=is_mergeable)
-        self.__document_model = document_model
-        self.__computation_proxy = computation.create_proxy()
-        self.__variable_index = computation.variables.index(variable)
-        self.__property_keys = kwargs.keys()
-        self.__properties = copy.deepcopy(kwargs)
-        self.initialize()
-
-    def close(self):
-        self.__properties = None
-        self.__computation_proxy.close()
-        self.__computation_proxy = None
-        self.__properties = None
-        self.__property_keys = None
-        super().close()
-
-    def perform(self):
-        computation = self.__computation_proxy.item
-        variable = computation.variables[self.__variable_index]
-        properties = self.__properties
-        self.__properties = {key: getattr(variable, key) for key in self.__property_keys}
-        for key, value in properties.items():
-            setattr(variable, key, value)
-
-    def _get_modified_state(self):
-        computation = self.__computation_proxy.item
-        return computation.modified_state, self.__document_model.modified_state
-
-    def _set_modified_state(self, modified_state) -> None:
-        computation = self.__computation_proxy.item
-        computation.modified_state, self.__document_model.modified_state = modified_state
-
-    def _compare_modified_states(self, state1, state2) -> bool:
-        # override to allow the undo command to track state; but only use part of the state for comparison
-        return state1[0] == state2[0]
-
-    def _undo(self):
-        computation = self.__computation_proxy.item
-        variable = computation.variables[self.__variable_index]
-        properties = self.__properties
-        self.__properties = {key: getattr(variable, key) for key in self.__property_keys}
-        for key, value in properties.items():
-            setattr(variable, key, value)
-
-    def can_merge(self, command: Undo.UndoableCommand) -> bool:
-        return isinstance(command, ChangeVariableCommand) and self.command_id and self.command_id == command.command_id and self.__computation_proxy.item == command.__computation_proxy.item and self.__variable_index == command.__variable_index
-
-
 def select_computation(document_model: DocumentModel.DocumentModel, display_item: DisplayItem.DisplayItem) -> typing.Optional[Symbolic.Computation]:
     if display_item:
         match_items = set()
@@ -297,7 +248,7 @@ def select_computation(document_model: DocumentModel.DocumentModel, display_item
 
 
 class ComputationModel:
-    """Represents a computation. Tracks a display specifier for changes to it and its computation content.
+    """Represents a computation. Tracks a computation for changes to it and its content.
 
     Provides read/write access to the computation_text via the property.
 
@@ -343,10 +294,12 @@ class ComputationModel:
     def set_display_item(self, display_item):
         self.__set_display_item(display_item)
 
-    def add_variable(self, name: str=None, value_type: str=None, value=None, value_default=None, value_min=None, value_max=None, control_type: str=None, specifier: dict=None, label: str=None) -> None:
+    def add_variable(self, name: str = None, value_type: str = None, value=None, value_default=None, value_min=None,
+                     value_max=None, control_type: str = None,
+                     specified_item: typing.Optional[Persistence.PersistentObject] = None, label: str = None) -> None:
         computation = self.computation
         if computation:
-            command = AddVariableCommand(self.document_controller.document_model, computation, name, value_type, value, value_default, value_min, value_max, control_type, specifier, label)
+            command = AddVariableCommand(self.document_controller.document_model, computation, name, value_type, value, value_default, value_min, value_max, control_type, specified_item, label)
             command.perform()
             self.document_controller.push_undo_command(command)
 
@@ -475,7 +428,7 @@ class ChangeVariableBinding(Binding.PropertyBinding):
 
         def set_value(value):
             if value != getattr(variable, property_name):
-                command = ChangeVariableCommand(document_controller.document_model, computation, self.source, **{self.__property_name: value})
+                command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, self.source, **{self.__property_name: value})
                 command.perform()
                 document_controller.push_undo_command(command)
 
@@ -655,22 +608,11 @@ class ComputationPanelSection:
 
             return column
 
-        def make_specifier_row(ui, variable: Symbolic.ComputationVariable, on_change_type_fn, on_remove_fn, *, include_secondary=False):
+        def make_specifier_row(ui, variable: Symbolic.ComputationVariable, on_change_type_fn, on_remove_fn):
             column = ui.create_column_widget()
 
             name_type_row = make_name_type_row(ui, variable, on_change_type_fn, on_remove_fn)
 
-            def make_uuid_row(label, binding_identifier):
-                uuid_text_edit = ui.create_line_edit_widget()
-                uuid_row = ui.create_row_widget()
-                uuid_row.add_spacing(8)
-                uuid_row.add(ui.create_label_widget(label))
-                uuid_row.add_spacing(8)
-                uuid_row.add(uuid_text_edit)
-                uuid_row.add_spacing(8)
-                uuid_text_edit.bind_text(ChangeVariableBinding(document_controller, computation, variable, binding_identifier))
-                return uuid_row
-
             label_text_edit = ui.create_line_edit_widget()
             label_text_edit.bind_text(ChangeVariableBinding(document_controller, computation, variable, "label"))
 
@@ -682,9 +624,6 @@ class ComputationPanelSection:
             display_row.add_stretch()
 
             column.add(name_type_row)
-            column.add(make_uuid_row(_("Data Item UUID"), "specifier_uuid_str"))
-            if include_secondary:
-                column.add(make_uuid_row(_("Region UUID"), "secondary_specifier_uuid_str"))
             column.add(display_row)
 
             return column
@@ -709,11 +648,11 @@ class ComputationPanelSection:
         twist_down_canvas_item.on_button_clicked = toggle
 
         def change_type(variable_type):
-            command = ChangeVariableCommand(document_controller.document_model, computation, variable, title=_("Remove Input Data Item"), variable_type=variable_type)
+            command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, variable, title=_("Remove Input Data Item"), variable_type=variable_type)
             command.perform()
             document_controller.push_undo_command(command)
 
-        def select_stack(stack, variable, specifier):
+        def select_stack(stack, variable):
             stack.remove_all()
             variable_type = variable.variable_type
             if variable_type == "boolean":
@@ -725,7 +664,7 @@ class ComputationPanelSection:
             elif variable_type == "string":
                 stack.add(make_string_row(ui, variable, None, change_type, on_remove))
             elif variable_type == "data_source":
-                stack.add(make_specifier_row(ui, variable, change_type, on_remove, include_secondary=True))
+                stack.add(make_specifier_row(ui, variable, change_type, on_remove))
             elif variable_type == "graphic":
                 stack.add(make_specifier_row(ui, variable, change_type, on_remove))
             else:
@@ -734,11 +673,11 @@ class ComputationPanelSection:
         def do_select_stack():
             # select stack will remove the inspector widgets, so delay it until the
             # current event (combo box changed) has finished by queueing it.
-            queue_task_fn(functools.partial(select_stack, stack, variable, variable.specifier))
+            queue_task_fn(functools.partial(select_stack, stack, variable))
 
         self.__variable_type_changed_event_listener = variable.variable_type_changed_event.listen(do_select_stack)
 
-        select_stack(stack, variable, variable.specifier)
+        select_stack(stack, variable)
 
         self.widget = section_widget
 
@@ -748,25 +687,19 @@ class ComputationPanelSection:
 
 
 def drop_mime_data(document_controller, computation: Symbolic.Computation, variable: Symbolic.ComputationVariable, mime_data: UserInterface.MimeData, x: int, y: int) -> typing.Optional[str]:
-    project = computation.project  # all variables/specifiers will go into the same project as the computation
     display_item, graphic = MimeTypes.mime_data_get_data_source(mime_data, document_controller.document_model)
     data_item = display_item.data_item if display_item else None
     if data_item:
-        variable_specifier = DataStructure.get_object_specifier(display_item.get_display_data_channel_for_data_item(data_item), project=project)
-        secondary_specifier = None
-        if graphic:
-            secondary_specifier = DataStructure.get_object_specifier(graphic, project=project)
-        properties = {"variable_type": "data_source", "secondary_specifier": secondary_specifier, "specifier": variable_specifier}
-        command = ChangeVariableCommand(document_controller.document_model, computation, variable, title=_("Set Input Data Source"), **properties)
+        properties = {"variable_type": "data_source", "secondary_specified_object": graphic, "specified_object": display_item.get_display_data_channel_for_data_item(data_item)}
+        command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, variable, title=_("Set Input Data Source"), **properties)
         command.perform()
         document_controller.push_undo_command(command)
         return "copy"
     display_item = MimeTypes.mime_data_get_display_item(mime_data, document_controller.document_model)
     data_item = display_item.data_item if display_item else None
     if data_item:
-        variable_specifier = DataStructure.get_object_specifier(display_item.get_display_data_channel_for_data_item(data_item), project=project)
-        properties = {"variable_type": "data_source", "secondary_specifier": dict(), "specifier": variable_specifier}
-        command = ChangeVariableCommand(document_controller.document_model, computation, variable, title=_("Set Input Data Source"), **properties)
+        properties = {"variable_type": "data_source", "secondary_specified_object": None, "specified_object": display_item.get_display_data_channel_for_data_item(data_item)}
+        command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, variable, title=_("Set Input Data Source"), **properties)
         command.perform()
         document_controller.push_undo_command(command)
         return "copy"
@@ -774,8 +707,7 @@ def drop_mime_data(document_controller, computation: Symbolic.Computation, varia
 
 
 def data_item_delete(document_controller, computation: Symbolic.Computation, variable: Symbolic.ComputationVariable) -> None:
-    variable_specifier = {"type": variable.variable_type, "version": 1, "uuid": str(uuid.uuid4())}
-    command = ChangeVariableCommand(document_controller.document_model, computation, variable, title=_("Remove Input Data Source"), specifier=variable_specifier)
+    command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, variable, title=_("Remove Input Data Source"), specified_object=None)
     command.perform()
     document_controller.push_undo_command(command)
 
@@ -807,7 +739,7 @@ def make_image_chooser(document_controller, computation: Symbolic.Computation, v
     data_item_chooser_widget.on_drop_mime_data = functools.partial(drop_mime_data, document_controller, computation, variable)
 
     def property_changed(key):
-        if key == "specifier":
+        if key == "specified_object":
             computation_input = computation.get_input(variable.name)
             data_item = computation_input.data_item if computation_input else None
             display_item = document_model.get_display_item_for_data_item(data_item)
@@ -895,8 +827,7 @@ class EditComputationDialog(Dialog.ActionDialog):
 
         def add_object_pressed():
             document_model = document_controller.document_model
-            object_specifier = document_model.get_object_specifier(document_model.data_items[0])
-            self.__computation_model.add_variable("".join([random.choice(string.ascii_lowercase) for _ in range(4)]), specifier=object_specifier)
+            self.__computation_model.add_variable("".join([random.choice(string.ascii_lowercase) for _ in range(4)]), specified_item=document_model.data_items[0])
 
         add_object_button.on_clicked = add_object_pressed
 
@@ -1153,10 +1084,16 @@ class DataStructureHandler(Observable.Observable):
         if base_entity_type:
             self.__entity_types = base_entity_type.subclasses
 
+            entity_info_list = [(DataStructure.DataStructure.entity_names[entity_type.entity_id],
+                                 DataStructure.DataStructure.entity_package_names[entity_type.entity_id])
+                                 for entity_type in self.__entity_types]
+
+            counts = collections.Counter([entity_info[0] for entity_info in entity_info_list])
+
             def name(entity_id: str) -> str:
                 entity_name = DataStructure.DataStructure.entity_names[entity_id]
                 entity_package_name = DataStructure.DataStructure.entity_package_names[entity_id]
-                return f"{entity_name} ({entity_package_name})"
+                return f"{entity_name} ({entity_package_name})" if counts[entity_name] > 1 else entity_name
 
             self.__entity_choices = [name(entity_type.entity_id) for entity_type in self.__entity_types] + ["-", _("None")]
             # configure the initial value
@@ -1202,14 +1139,14 @@ class VariableHandler:
         self.float_str_converter = Converter.FloatToStringConverter()
         self.int_str_converter = Converter.IntegerToStringConverter()
         self.property_changed_event = Event.Event()
-        self.__specifier_changed_listener = variable.property_changed_event.listen(self.__variable_property_changed)
+        self.__variable_property_changed_listener = variable.property_changed_event.listen(self.__variable_property_changed)
 
     def close(self) -> None:
-        self.__specifier_changed_listener.close()
-        self.__specifier_changed_listener = None
+        self.__variable_property_changed_listener.close()
+        self.__variable_property_changed_listener = None
 
     def __variable_property_changed(self, property_name: str) -> None:
-        if property_name in ("specifier", "secondary_specifier"):
+        if property_name in ("specified_object", "secondary_specified_object"):
             self.property_changed_event.fire("display_item")
         elif property_name == "value":
             self.property_changed_event.fire("variable_value")
@@ -1225,7 +1162,7 @@ class VariableHandler:
         computation = self.computation
         variable = self.variable
         if value != variable.value:
-            command = ChangeVariableCommand(document_controller.document_model, computation, variable, value=value)
+            command = Inspector.ChangeComputationVariableCommand(document_controller.document_model, computation, variable, value=value)
             command.perform()
             document_controller.push_undo_command(command)
 
@@ -1341,11 +1278,11 @@ class VariableHandler:
         elif variable.variable_type == "graphic":
             return u.create_column(label, u.create_component_instance("graphic"), spacing=8)
         elif variable.variable_type == "structure":
-            return u.create_column(label, u.create_component_instance("structure"), spacing=8)
+            return u.create_column(u.create_component_instance("structure"), spacing=8)
         elif variable.bound_items_model:
             return u.create_column(u.create_column(items="variable.bound_items_model.items", item_component_id="graphic_item", spacing=8), spacing=8)
         else:
-            return u.create_column(label, u.create_label(text=_("Missing") + " " + f"[{variable.variable_type} {variable.specifier}]"), spacing=8)
+            return u.create_column(label, u.create_label(text=_("Missing") + " " + f"[{variable.variable_type}]"), spacing=8)
 
 
 class ResultHandler:
