@@ -279,7 +279,7 @@ class ImageCanvasItemDelegate:
 
     def cursor_changed(self, pos): ...
 
-    def update_display_properties(self, display_properties: typing.Mapping, display_layers: typing.Sequence[typing.Mapping]) -> None: ...
+    def update_display_properties(self, display_properties: typing.Mapping) -> None: ...
 
     def update_display_data_channel_properties(self, display_data_channel_properties: typing.Mapping) -> None: ...
 
@@ -494,7 +494,7 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
         self.__display_values = display_values_list[0] if display_values_list else None
         self.__display_values_dirty = True
 
-    def update_display_properties(self, display_calibration_info, display_properties: typing.Mapping, display_layers: typing.Sequence[typing.Mapping]) -> None:
+    def update_display_properties_and_layers(self, display_calibration_info, display_properties: typing.Mapping, display_layers: typing.Sequence[typing.Mapping]) -> None:
         # threadsafe
         data_and_metadata = self.__display_values.data_and_metadata if self.__display_values else None
         if data_and_metadata:
@@ -716,7 +716,7 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
             part_spec = specific_part_spec if specific_part_spec is not None else part_specs[-1] if len(part_specs) > 0 else None
             if part_spec is not None:
                 graphic_index, graphic, already_selected, part = part_spec
-                part = part if specific_part_spec is not None else "all"
+                part = part if specific_part_spec is not None else part_spec[-1]
                 # select item and prepare for drag
                 self.graphic_drag_item_was_selected = already_selected
                 if not self.graphic_drag_item_was_selected:
@@ -1241,9 +1241,9 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
         if self.__data_shape is not None:
             # configure the bitmap canvas item
             display_values = self.__display_values
-            display_data = display_values.display_data_and_metadata
-            if display_data and display_data.data.dtype == numpy.float32:
-                display_range = display_values.display_range
+            display_data = display_values.adjusted_data_and_metadata
+            if display_data and display_data.data_dtype == numpy.float32:
+                display_range = display_values.transformed_display_range
                 color_map_data = display_values.color_map_data
                 display_values.finalize()
                 if color_map_data is not None:
@@ -1257,21 +1257,6 @@ class ImageCanvasItem(CanvasItem.LayerCanvasItem):
             else:
                 data_rgba = display_values.display_rgba
                 display_values.finalize()
-                if False:
-                    # the next section does gaussian blur for image decimation in the case where the destination rectangle
-                    # is smaller than the source. this results in lower performance, but higher quality display.
-                    height_ratio = (self.__bitmap_canvas_item.canvas_size.height / data_rgba.shape[0]) if data_rgba is not None and data_rgba.shape[0] > 0 else 1
-                    if height_ratio < 1:
-                        sigma = 0.5 * ((1.0 / height_ratio) - 1.0)
-                        data_rgba_copy = numpy.empty_like(data_rgba)
-                        data_rgba_u8_view = data_rgba.view(numpy.uint8).reshape(data_rgba.shape + (-1, ))
-                        data_rgba_u8_copy_view = data_rgba_copy.view(numpy.uint8).reshape(data_rgba_copy.shape + (-1, ))
-                        data_rgba_u8_copy_view[..., 0] = scipy.ndimage.gaussian_filter(data_rgba_u8_view[..., 0], sigma=sigma)
-                        data_rgba_u8_copy_view[..., 1] = scipy.ndimage.gaussian_filter(data_rgba_u8_view[..., 1], sigma=sigma)
-                        data_rgba_u8_copy_view[..., 2] = scipy.ndimage.gaussian_filter(data_rgba_u8_view[..., 2], sigma=sigma)
-                        if data_rgba_u8_view.shape[-1] == 4:
-                            data_rgba_u8_copy_view[..., 3] = data_rgba_u8_view[..., 3]
-                        data_rgba = data_rgba_copy
                 self.__bitmap_canvas_item.set_rgba_bitmap_data(data_rgba, trigger_update=False)
             self.__timestamp_canvas_item.timestamp = display_values.display_rgba_timestamp if self.__display_latency else None
 
