@@ -278,7 +278,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
     def update_display_values(self, display_values_list) -> None:
         self.__display_values_list = display_values_list
 
-    def update_display_properties(self, display_calibration_info, display_properties: typing.Mapping, display_layers: typing.Sequence[typing.Mapping]) -> None:
+    def update_display_properties_and_layers(self, display_calibration_info, display_properties: typing.Mapping, display_layers: typing.Sequence[typing.Mapping]) -> None:
         """Update the display values. Called from display panel.
 
         This method saves the display values and data and triggers an update. It should be as fast as possible.
@@ -505,48 +505,45 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
 
             colors = ('#1E90FF', "#F00", "#0F0", "#00F", "#FF0", "#0FF", "#F0F", "#888", "#800", "#080", "#008", "#CCC", "#880", "#088", "#808", "#964B00")
 
-            display_layers = self.__display_layers
+            display_layers = list(self.__display_layers)
 
             if len(display_layers) == 0:
                 index = 0
                 for scalar_index, scalar_xdata in enumerate(scalar_xdata_list):
                     if scalar_xdata and scalar_xdata.is_data_1d:
-                        if index < 16:
+                        if index < len(self.__line_graph_stack.canvas_items):
                             display_layers.append({"fill_color": colors[index] if index == 0 else None, "stroke_color": colors[index] if index > 0 else None, "data_index": scalar_index})
                             index += 1
                     if scalar_xdata and scalar_xdata.is_data_2d:
-                        for row in range(min(scalar_xdata.data_shape[-1], 16)):
-                            if index < 16:
+                        for row in range(min(scalar_xdata.data_shape[-1], len(self.__line_graph_stack.canvas_items))):
+                            if index < len(self.__line_graph_stack.canvas_items):
                                 display_layers.append({"fill_color": colors[index] if index == 0 else None, "stroke_color": colors[index] if index > 0 else None, "data_index": scalar_index, "data_row": row})
                                 index += 1
 
-            display_layer_count = len(display_layers)
-
             self.___has_valid_drawn_graph_data = False
 
-            for index, display_layer in enumerate(display_layers):
-                if index < 16:
-                    fill_color = display_layer.get("fill_color")
-                    stroke_color = display_layer.get("stroke_color")
-                    data_index = display_layer.get("data_index", 0)
-                    data_row = display_layer.get("data_row", 0)
-                    if 0 <= data_index < len(scalar_xdata_list):
-                        scalar_xdata = scalar_xdata_list[data_index]
-                        if scalar_xdata:
-                            data_row = max(0, min(scalar_xdata.dimensional_shape[0] - 1, data_row))
-                            intensity_calibration = scalar_xdata.intensity_calibration
-                            displayed_dimensional_calibration = scalar_xdata.dimensional_calibrations[-1]
-                            if scalar_xdata.is_data_2d:
-                                scalar_data = scalar_xdata.data[data_row:data_row + 1, :].reshape((scalar_xdata.dimensional_shape[-1],))
-                                scalar_xdata = DataAndMetadata.new_data_and_metadata(scalar_data, intensity_calibration, [displayed_dimensional_calibration])
-                        line_graph_canvas_item = self.__line_graph_stack.canvas_items[display_layer_count - (index + 1)]
-                        line_graph_canvas_item.set_fill_color(fill_color)
-                        line_graph_canvas_item.set_stroke_color(stroke_color)
-                        line_graph_canvas_item.set_axes(axes)
-                        line_graph_canvas_item.set_uncalibrated_xdata(scalar_xdata)
-                        self.___has_valid_drawn_graph_data = scalar_xdata is not None
+            for index, display_layer in enumerate(self.__display_layers[0:len(self.__line_graph_stack.canvas_items)]):
+                fill_color = display_layer.get("fill_color")
+                stroke_color = display_layer.get("stroke_color")
+                data_index = display_layer.get("data_index", 0)
+                data_row = display_layer.get("data_row", 0)
+                if 0 <= data_index < len(scalar_xdata_list):
+                    scalar_xdata = scalar_xdata_list[data_index]
+                    if scalar_xdata:
+                        data_row = max(0, min(scalar_xdata.dimensional_shape[0] - 1, data_row))
+                        intensity_calibration = scalar_xdata.intensity_calibration
+                        displayed_dimensional_calibration = scalar_xdata.dimensional_calibrations[-1]
+                        if scalar_xdata.is_data_2d:
+                            scalar_data = scalar_xdata.data[data_row:data_row + 1, :].reshape((scalar_xdata.dimensional_shape[-1],))
+                            scalar_xdata = DataAndMetadata.new_data_and_metadata(scalar_data, intensity_calibration, [displayed_dimensional_calibration])
+                    line_graph_canvas_item = self.__line_graph_stack.canvas_items[index]
+                    line_graph_canvas_item.set_fill_color(fill_color)
+                    line_graph_canvas_item.set_stroke_color(stroke_color)
+                    line_graph_canvas_item.set_axes(axes)
+                    line_graph_canvas_item.set_uncalibrated_xdata(scalar_xdata)
+                    self.___has_valid_drawn_graph_data = scalar_xdata is not None
 
-            for index in range(len(display_layers), 16):
+            for index in range(len(display_layers), len(self.__line_graph_stack.canvas_items)):
                 line_graph_canvas_item = self.__line_graph_stack.canvas_items[index]
                 line_graph_canvas_item.set_axes(None)
                 line_graph_canvas_item.set_uncalibrated_xdata(None)
@@ -554,7 +551,7 @@ class LinePlotCanvasItem(CanvasItem.LayerCanvasItem):
             legend_position = self.__legend_position
             LegendEntry = collections.namedtuple("LegendEntry", ["label", "fill_color", "stroke_color"])
             legend_entries = list()
-            for index, display_layer in enumerate(self.__display_layers):
+            for index, display_layer in enumerate(self.__display_layers[0:len(self.__line_graph_stack.canvas_items)]):
                 data_index = display_layer.get("data_index", None)
                 data_row = display_layer.get("data_row", None)
                 label = display_layer.get("label", str())
