@@ -1113,7 +1113,9 @@ class DataItem(Observable.Observable, Persistence.PersistentObject):
                     assert self.__data_and_metadata.data_shape == data_metadata.data_shape
                     assert self.__data_and_metadata.data_dtype == data_metadata.data_dtype
                     assert self.__data_and_metadata.data_dtype == data_and_metadata.data_dtype
-                    self.__data_and_metadata.data[dst] = data_and_metadata.data[src]
+                    self.__data_and_metadata.data[tuple(dst)] = data_and_metadata.data[tuple(src)]
+                    # set data_shape as a way to update 'modified' property
+                    self._set_persistent_property_value("data_shape", self.__data_and_metadata.data_shape)
                     if self.persistent_object_context and not self.is_write_delayed:
                         self.write_external_data("data", self.__data_and_metadata.data)
                         self.__data_and_metadata.unloadable = True
@@ -1300,6 +1302,16 @@ class DataSource:
             return self.data_item.xdata
         return None
 
+    @property
+    def element_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        display_data_channel = self.__display_data_channel
+        if display_data_channel:
+            if self.__xdata is not None:
+                return Core.function_convert_to_scalar(self.xdata, display_data_channel.complex_display_type)
+            else:
+                return display_data_channel.get_calculated_display_values().element_data_and_metadata
+        return None
+
     @property
     def display_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
         display_data_channel = self.__display_data_channel
@@ -1307,26 +1319,83 @@ class DataSource:
             if self.__xdata is not None:
                 return Core.function_convert_to_scalar(self.xdata, display_data_channel.complex_display_type)
             else:
-                return display_data_channel.get_calculated_display_values(True).display_data_and_metadata
+                return display_data_channel.get_calculated_display_values().display_data_and_metadata
         return None
 
     @property
-    def cropped_display_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
-        data_item = self.data_item
-        if data_item:
-            displayed_xdata = self.display_xdata
-            graphic = self.__graphic
-            if graphic:
-                if hasattr(graphic, "bounds") and displayed_xdata.is_data_2d:
-                    if graphic.rotation:
-                        return Core.function_crop_rotated(displayed_xdata, graphic.bounds, graphic.rotation)
-                    else:
-                        return Core.function_crop(displayed_xdata, graphic.bounds)
-                if hasattr(graphic, "interval") and displayed_xdata.is_data_1d:
-                    return Core.function_crop_interval(displayed_xdata, graphic.interval)
-            return displayed_xdata
+    def display_rgba(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        display_data_channel = self.__display_data_channel
+        if display_data_channel:
+            if self.__xdata is not None:
+                return self.xdata
+            else:
+                display_rgba = display_data_channel.get_calculated_display_values().display_rgba
+                return DataAndMetadata.new_data_and_metadata(Image.get_byte_view(display_rgba)) if display_rgba is not None else None
         return None
 
+    @property
+    def normalized_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        display_data_channel = self.__display_data_channel
+        if display_data_channel:
+            if self.__xdata is not None:
+                return Core.function_rescale(self.display_xdata, (0, 1))
+            else:
+                return display_data_channel.get_calculated_display_values().normalized_data_and_metadata
+        return None
+
+    @property
+    def adjusted_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        display_data_channel = self.__display_data_channel
+        if display_data_channel:
+            if self.__xdata is not None:
+                return self.normalized_xdata
+            else:
+                return display_data_channel.get_calculated_display_values().adjusted_data_and_metadata
+        return None
+
+    @property
+    def transformed_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        display_data_channel = self.__display_data_channel
+        if display_data_channel:
+            if self.__xdata is not None:
+                return self.normalized_xdata
+            else:
+                return display_data_channel.get_calculated_display_values().transformed_data_and_metadata
+        return None
+
+    def __cropped_xdata(self, xdata: typing.Optional[DataAndMetadata.DataAndMetadata]) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        data_item = self.data_item
+        graphic = self.__graphic
+        if data_item and graphic:
+            if hasattr(graphic, "bounds") and xdata.is_data_2d:
+                if graphic.rotation:
+                    return Core.function_crop_rotated(xdata, graphic.bounds, graphic.rotation)
+                else:
+                    return Core.function_crop(xdata, graphic.bounds)
+            if hasattr(graphic, "interval") and xdata.is_data_1d:
+                return Core.function_crop_interval(xdata, graphic.interval)
+        return xdata
+
+    @property
+    def cropped_element_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__cropped_xdata(self.element_xdata)
+
+    @property
+    def cropped_display_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__cropped_xdata(self.display_xdata)
+
+    @property
+    def cropped_normalized_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__cropped_xdata(self.normalized_xdata)
+
+    @property
+    def cropped_adjusted_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__cropped_xdata(self.adjusted_xdata)
+
+    @property
+    def cropped_transformed_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
+        return self.__cropped_xdata(self.transformed_xdata)
+
     @property
     def cropped_xdata(self) -> typing.Optional[DataAndMetadata.DataAndMetadata]:
         data_item = self.data_item
