@@ -191,7 +191,7 @@ def create_data_item_from_data_element(data_element, data_file_path=None):
     large_format = data_element.get("large_format")
     if large_format is None:
         data = data_element.get("data")
-        large_format = len(data.shape) > 2 if data is not None else False
+        large_format = len(data.shape) > 2 and data.dtype != numpy.uint8 if data is not None else False
     data_item = DataItem.DataItem(item_uuid=uuid_, large_format=large_format)
     update_data_item_from_data_element(data_item, data_element, data_file_path)
     return data_item
@@ -491,7 +491,7 @@ class StandardImportExportHandler(ImportExportHandler):
         return len(data_and_metadata.dimensional_shape) == 2
 
     def write_display_item(self, ui, display_item: DisplayItem.DisplayItem, path_str: str, extension: str) -> None:
-        data = display_item.display_data_channel.get_calculated_display_values(True).display_rgba  # export the display rather than the data for these types
+        data = display_item.display_data_channel.get_calculated_display_values().display_rgba  # export the display rather than the data for these types
         if data is not None:
             ui.save_rgba_data_to_file(data, path_str, extension)
 
