@@ -82,6 +82,9 @@ class ProjectReference(Observable.Observable, Persistence.PersistentObject):
 
     def read_from_dict(self, properties):
         super().read_from_dict(properties)
+        self.establish_last_used()
+
+    def establish_last_used(self) -> None:
         if not self.last_used:
             self.last_used = self._get_last_used()
 
@@ -117,6 +120,13 @@ class ProjectReference(Observable.Observable, Persistence.PersistentObject):
             return pathlib.Path(project_reference_parts[-1]).stem
         return _("Untited")
 
+    @property
+    def path(self) -> pathlib.Path:
+        project_reference_parts = self.project_reference_parts
+        if project_reference_parts:
+            return pathlib.Path(*project_reference_parts)
+        return pathlib.Path()
+
     @property
     def project_reference_parts(self) -> typing.Tuple[str]:
         raise NotImplementedError()
@@ -473,18 +483,21 @@ class Profile(Observable.Observable, Persistence.PersistentObject):
         project_reference = IndexProjectReference()
         project_reference.project_path = project_path
         project_reference.project_uuid = project_reference.read_project_uuid(self.profile_context)
+        project_reference.establish_last_used()
         return self.add_project_reference(project_reference, load)
 
     def add_project_folder(self, project_folder_path: pathlib.Path, load: bool = True) -> ProjectReference:
         project_reference = FolderProjectReference()
         project_reference.project_folder_path = project_folder_path
         project_reference.project_uuid = project_reference.read_project_uuid(self.profile_context)
+        project_reference.establish_last_used()
         return self.add_project_reference(project_reference, load)
 
     def add_project_memory(self, _uuid: uuid.UUID = None, load: bool = True) -> ProjectReference:
         assert callable(project_reference_factory_hook)
         project_reference = project_reference_factory_hook("project_memory")
         project_reference.project_uuid = _uuid or uuid.uuid4()
+        project_reference.establish_last_used()
         return self.add_project_reference(project_reference, load)
 
     def upgrade(self, project_reference: ProjectReference) -> typing.Optional[ProjectReference]:
