@@ -16,7 +16,6 @@ import threading
 import time
 import typing
 import uuid
-import weakref
 
 # third party libraries
 import numpy
@@ -68,8 +67,8 @@ class ComputationOutput(Observable.Observable, Persistence.PersistentObject):
         self.define_type("output")
         self.define_property("name", name, changed=self.__property_changed)
         self.define_property("label", label if label else name, hidden=True, changed=self.__property_changed)
-        self.define_property("specifier", specifier, changed=self.__property_changed)
-        self.define_property("specifiers", specifiers, changed=self.__property_changed)
+        self.define_property("specifier", specifier, hidden=True, changed=self.__property_changed)
+        self.define_property("specifiers", specifiers, hidden=True, changed=self.__property_changed)
         self.needs_rebind_event = Event.Event()  # an event to be fired when the computation needs to rebind
         self.bound_item = None
         self.__needs_rebind_event_listeners = list()
@@ -81,6 +80,22 @@ class ComputationOutput(Observable.Observable, Persistence.PersistentObject):
         self.__needs_rebind_event_listeners = None
         super().close()
 
+    @property
+    def _specifier(self):
+        return self._get_persistent_property_value("specifier")
+
+    @_specifier.setter
+    def _specifier(self, value):
+        self._set_persistent_property_value("specifier", value)
+
+    @property
+    def _specifiers(self):
+        return self._get_persistent_property_value("specifiers")
+
+    @_specifiers.setter
+    def _specifiers(self, value):
+        self._set_persistent_property_value("specifiers", value)
+
     def __property_changed(self, name, value):
         self.notify_property_changed(name)
         if name in ("specifier", "specifiers"):
@@ -92,12 +107,12 @@ class ComputationOutput(Observable.Observable, Persistence.PersistentObject):
         self.notify_property_changed("bound_item")
 
     def bind(self, resolve_object_specifier_fn):
-        if self.specifier:
-            self.bound_item = resolve_object_specifier_fn(self.specifier)
+        if self._specifier:
+            self.bound_item = resolve_object_specifier_fn(self._specifier)
             if self.bound_item:
                 self.__needs_rebind_event_listeners.append(self.bound_item.needs_rebind_event.listen(self.__unbind))
-        elif self.specifiers is not None:
-            bound_items = [resolve_object_specifier_fn(specifier) for specifier in self.specifiers]
+        elif self._specifiers is not None:
+            bound_items = [resolve_object_specifier_fn(specifier) for specifier in self._specifiers]
             bound_items = [bound_item for bound_item in bound_items if bound_item is not None]
             for bound_item in bound_items:
                 self.__needs_rebind_event_listeners.append(bound_item.needs_rebind_event.listen(self.__unbind))
@@ -167,8 +182,8 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
         self.define_property("value_default", value_default, changed=self.__property_changed, reader=self.__value_reader, writer=self.__value_writer)
         self.define_property("value_min", value_min, changed=self.__property_changed, reader=self.__value_reader, writer=self.__value_writer)
         self.define_property("value_max", value_max, changed=self.__property_changed, reader=self.__value_reader, writer=self.__value_writer)
-        self.define_property("specifier", specifier, changed=self.__property_changed)
-        self.define_property("secondary_specifier", secondary_specifier, changed=self.__property_changed)
+        self.define_property("specifier", specifier, hidden=True, changed=self.__property_changed)
+        self.define_property("secondary_specifier", secondary_specifier, hidden=True, changed=self.__property_changed)
         self.define_property("property_name", property_name, changed=self.__property_changed)
         self.define_property("control_type", control_type, changed=self.__property_changed)
         item_specifiers = [DataStructure.get_object_specifier(item.item, item.type, allow_partial=False) if item else None for item in items] if items is not None else None
@@ -196,7 +211,7 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
         super().close()
 
     def __repr__(self):
-        return "{} ({} {} {} {} {})".format(super().__repr__(), self.name, self.label, self.value, self.specifier, self.secondary_specifier)
+        return "{} ({} {} {} {} {})".format(super().__repr__(), self.name, self.label, self.value, self._specifier, self._secondary_specifier)
 
     def read_from_dict(self, properties: dict) -> None:
         # used for persistence
@@ -209,6 +224,22 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
         # used for persistence. left here since read_from_dict is defined.
         return super().write_to_dict()
 
+    @property
+    def _specifier(self):
+        return self._get_persistent_property_value("specifier")
+
+    @_specifier.setter
+    def _specifier(self, value):
+        self._set_persistent_property_value("specifier", value)
+
+    @property
+    def _secondary_specifier(self):
+        return self._get_persistent_property_value("secondary_specifier")
+
+    @_secondary_specifier.setter
+    def _secondary_specifier(self, value):
+        self._set_persistent_property_value("secondary_specifier", value)
+
     def connect_items(self, bound_items: typing.List) -> None:
         self.disconnect_items()
 
@@ -248,13 +279,13 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
 
     def save_properties(self):
         # used for undo
-        return self.value, self.specifier, self.secondary_specifier
+        return self.value, self._specifier, self._secondary_specifier
 
     def restore_properties(self, properties):
         # used for undo
         self.value = properties[0]
-        self.specifier = properties[1]
-        self.secondary_specifier = properties[2]
+        self._specifier = properties[1]
+        self._secondary_specifier = properties[2]
 
     def __value_reader(self, persistent_property, properties):
         value_type = self.value_type
@@ -287,7 +318,7 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
                 properties[persistent_property.key] = str(value)
 
     @property
-    def variable_specifier(self) -> dict:
+    def _variable_specifier(self) -> dict:
         """Return the variable specifier for this variable.
 
         The specifier can be used to lookup the value of this variable in a computation context.
@@ -295,7 +326,29 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
         if self.value_type is not None:
             return {"type": "variable", "version": 1, "uuid": str(self.uuid), "x-name": self.name, "x-value": self.value}
         else:
-            return self.specifier
+            return self._specifier
+
+    @property
+    def specified_object(self) -> typing.Optional[Persistence.PersistentObject]:
+        return self.__bound_item.value
+
+    @specified_object.setter
+    def specified_object(self, value: typing.Optional[Persistence.PersistentObject]) -> None:
+        if value:
+            self._specifier = DataStructure.get_object_specifier(value)
+        else:
+            self._specifier = {"type": "data_source", "version": 1, "uuid": str(uuid.uuid4())}
+
+    @property
+    def secondary_specified_object(self) -> typing.Optional[Persistence.PersistentObject]:
+        return getattr(self.__bound_item, "_graphic", None)
+
+    @secondary_specified_object.setter
+    def secondary_specified_object(self, value: typing.Optional[Persistence.PersistentObject]) -> None:
+        if value:
+            self._secondary_specifier = DataStructure.get_object_specifier(value)
+        else:
+            self._secondary_specifier = None
 
     @property
     def bound_variable(self) -> "BoundItemBase":
@@ -377,9 +430,9 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
         if name in ["name", "label"]:
             self.notify_property_changed("display_label")
         if name in ("specifier"):
-            self.notify_property_changed("specifier_uuid_str")
+            self.notify_property_changed("specified_object")
         if name in ("secondary_specifier"):
-            self.notify_property_changed("secondary_specifier_uuid_str")
+            self.notify_property_changed("secondary_specified_object")
         # send out the changed event
         self.changed_event.fire()
         # finally send out the rebuild event for the inspectors
@@ -401,20 +454,20 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
     def variable_type(self) -> typing.Optional[str]:
         if self.value_type is not None:
             return self.value_type
-        elif self.specifier is not None:
-            specifier_type = self.specifier.get("type")
-            specifier_property = self.specifier.get("property")
+        elif self._specifier is not None:
+            specifier_type = self._specifier.get("type")
+            specifier_property = self._specifier.get("property")
             return specifier_property or specifier_type
         return None
 
-    data_item_types = ("data_item", "data", "display_data")  # used for backward compatibility
+    data_item_types = ("data_item", "data", "display_data", "data_source")  # used for backward compatibility
 
     @variable_type.setter
     def variable_type(self, value_type: str) -> None:
         if value_type != self.variable_type:
             if value_type in ("boolean", "integral", "real", "complex", "string"):
-                self.specifier = None
-                self.secondary_specifier = None
+                self._specifier = None
+                self._secondary_specifier = None
                 self.value_type = value_type
                 self.control_type = self.control_type_default(value_type)
                 if value_type == "boolean":
@@ -435,7 +488,7 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
                 self.value_default = None
                 self.value_min = None
                 self.value_max = None
-                specifier = self.specifier or {"version": 1}
+                specifier = self._specifier or {"version": 1}
                 if not specifier.get("type") in ComputationVariable.data_item_types:
                     specifier.pop("uuid", None)
                 specifier["type"] = "data_source"
@@ -443,53 +496,53 @@ class ComputationVariable(Observable.Observable, Persistence.PersistentObject):
                     specifier["property"] = value_type
                 else:
                     specifier.pop("property", None)
-                self.specifier = specifier
-                self.secondary_specifier = self.secondary_specifier or {"version": 1}
+                self._specifier = specifier
+                self._secondary_specifier = self._secondary_specifier or {"version": 1}
             elif value_type in ("graphic"):
                 self.value_type = None
                 self.control_type = None
                 self.value_default = None
                 self.value_min = None
                 self.value_max = None
-                specifier = self.specifier or {"version": 1}
+                specifier = self._specifier or {"version": 1}
                 specifier["type"] = value_type
                 specifier.pop("uuid", None)
                 specifier.pop("property", None)
-                self.specifier = specifier
-                self.secondary_specifier = None
+                self._specifier = specifier
+                self._secondary_specifier = None
             self.variable_type_changed_event.fire()
 
     @property
-    def specifier_uuid_str(self):
-        return self.specifier.get("uuid") if self.specifier else None
+    def _specifier_uuid_str(self):
+        return self._specifier.get("uuid") if self._specifier else None
 
-    @specifier_uuid_str.setter
-    def specifier_uuid_str(self, value):
+    @_specifier_uuid_str.setter
+    def _specifier_uuid_str(self, value):
         converter = Converter.UuidToStringConverter()
         value = converter.convert(converter.convert_back(value))
-        if self.specifier_uuid_str != value and self.specifier:
-            specifier = self.specifier
+        if self._specifier_uuid_str != value and self._specifier:
+            specifier = self._specifier
             if value:
                 specifier["uuid"] = value
             else:
                 specifier.pop("uuid", None)
-            self.specifier = specifier
+            self._specifier = specifier
 
     @property
-    def secondary_specifier_uuid_str(self):
-        return self.secondary_specifier.get("uuid") if self.secondary_specifier else None
+    def _secondary_specifier_uuid_str(self):
+        return self._secondary_specifier.get("uuid") if self._secondary_specifier else None
 
-    @secondary_specifier_uuid_str.setter
-    def secondary_specifier_uuid_str(self, value):
+    @_secondary_specifier_uuid_str.setter
+    def _secondary_specifier_uuid_str(self, value):
         converter = Converter.UuidToStringConverter()
         value = converter.convert(converter.convert_back(value))
-        if self.secondary_specifier_uuid_str != value and self.secondary_specifier:
-            secondary_specifier = self.secondary_specifier
+        if self._secondary_specifier_uuid_str != value and self._secondary_specifier:
+            secondary_specifier = self._secondary_specifier
             if value:
                 secondary_specifier["uuid"] = value
             else:
                 secondary_specifier.pop("uuid", None)
-            self.secondary_specifier = secondary_specifier
+            self._secondary_specifier = secondary_specifier
 
     @property
     def display_label(self) -> str:
@@ -565,10 +618,10 @@ class BoundData(BoundItemBase):
             if self.__data_changed_event_listener:
                 self.__data_changed_event_listener.close()
                 self.__data_changed_event_listener = None
-            object = self._object
-            if object:
-                self.__data_changed_event_listener = object.data_changed_event.listen(self.changed_event.fire)
-            self.valid = object is not None
+            item = self._item
+            if item:
+                self.__data_changed_event_listener = item.data_changed_event.listen(self.changed_event.fire)
+            self.valid = item is not None
 
         def item_registered(item):
             maintain_data_source()
@@ -592,14 +645,14 @@ class BoundData(BoundItemBase):
 
     @property
     def base_objects(self):
-        return {self._object}
+        return {self._item}
 
     @property
     def value(self):
-        return self._object.xdata
+        return self._item.xdata
 
     @property
-    def _object(self):
+    def _item(self):
         item = self.__item_proxy.item
         if isinstance(item, DisplayItem.DisplayDataChannel):
             item = item.data_item
@@ -778,7 +831,7 @@ class BoundDisplayData(BoundDisplayDataChannelBase):
 
     @property
     def value(self):
-        return self._display_data_channel.get_calculated_display_values(True).display_data_and_metadata if self._display_data_channel else None
+        return self._display_data_channel.get_calculated_display_values().display_data_and_metadata if self._display_data_channel else None
 
 
 class BoundCroppedData(BoundDisplayDataChannelBase):
@@ -799,7 +852,7 @@ class BoundCroppedDisplayData(BoundDisplayDataChannelBase):
 
     @property
     def value(self):
-        xdata = self._display_data_channel.get_calculated_display_values(True).display_data_and_metadata
+        xdata = self._display_data_channel.get_calculated_display_values().display_data_and_metadata
         graphic = self._graphic
         if graphic:
             if hasattr(graphic, "bounds"):
@@ -817,7 +870,7 @@ class BoundFilterData(BoundDisplayDataChannelBase):
         # no display item is a special case for cascade removing graphics from computations. ugh.
         # see test_new_computation_becomes_unresolved_when_xdata_input_is_removed_from_document.
         if display_item:
-            shape = self._display_data_channel.get_calculated_display_values(True).display_data_and_metadata.data_shape
+            shape = self._display_data_channel.get_calculated_display_values().display_data_and_metadata.data_shape
             calibrated_origin = Geometry.FloatPoint(y=display_item.datum_calibrations[0].convert_from_calibrated_value(0.0),
                                                     x=display_item.datum_calibrations[1].convert_from_calibrated_value(0.0))
             mask = DataItem.create_mask_data(display_item.graphics, shape, calibrated_origin)
@@ -1094,6 +1147,7 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
         self.define_property("processing_id")  # see note above
         self.define_relationship("variables", variable_factory)
         self.define_relationship("results", result_factory)
+        self.attributes = dict()  # not persistent, defined by underlying class
         self.__source_proxy = self.create_item_proxy()
         self.__variable_changed_event_listeners = dict()
         self.__variable_needs_rebind_event_listeners = dict()
@@ -1195,6 +1249,13 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
     def label(self, value: str) -> None:
         self._set_persistent_property_value("label", value)
 
+    def get_computation_attribute(self, attribute: str, default: typing.Any = None) -> typing.Any:
+        """Returns the attribute for the computation."""
+        compute_class = _computation_types.get(self.processing_id)
+        if compute_class:
+            return getattr(compute_class, "attributes", dict()).get(attribute, default)
+        return default
+
     @property
     def error_text(self) -> typing.Optional[str]:
         return self._get_persistent_property_value("error_text")
@@ -1232,8 +1293,14 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
         self.computation_mutated_event.fire()
         self.needs_update = True
 
-    def create_variable(self, name: str=None, value_type: str=None, value=None, value_default=None, value_min=None, value_max=None, control_type: str=None, specifier: dict=None, label: str=None) -> ComputationVariable:
-        variable = ComputationVariable(name, value_type=value_type, value=value, value_default=value_default, value_min=value_min, value_max=value_max, control_type=control_type, specifier=specifier, label=label)
+    def create_variable(self, name: str = None, value_type: str = None, value=None, value_default=None, value_min=None,
+                        value_max=None, control_type: str = None,
+                        specified_item: typing.Optional[Persistence.PersistentObject] = None,
+                        label: str = None) -> ComputationVariable:
+        specifier = DataStructure.get_object_specifier(specified_item)
+        variable = ComputationVariable(name, value_type=value_type, value=value, value_default=value_default,
+                                       value_min=value_min, value_max=value_max, control_type=control_type,
+                                       specifier=specifier, label=label)
         self.add_variable(variable)
         return variable
 
@@ -1347,7 +1414,7 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
                     bound_item = BoundDataStructure(project, specifier, property_name)
                 elif specifier_type == "graphic":
                     bound_item = BoundGraphic(project, specifier, property_name)
-        elif variable.specifier is None:
+        elif variable._specifier is None:
             bound_item = variable.bound_variable
         if bound_item and not bound_item.valid:
             bound_item.close()
@@ -1398,9 +1465,9 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
             else:
                 is_resolved = False
         for result in self.results:
-            if result.specifier and not result.bound_item:
+            if result._specifier and not result.bound_item:
                 is_resolved = False
-            if result.specifiers and not all(result.bound_item):
+            if result._specifiers and not all(result.bound_item):
                 is_resolved = False
         return kwargs, is_resolved
 
@@ -1485,12 +1552,12 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
 
     @property
     def is_resolved(self):
-        if not all(not v.specifier or v.bound_item for v in self.variables):
+        if not all(not v._specifier or v.bound_item for v in self.variables):
             return False
         for result in self.results:
-            if result.specifier and not result.bound_item:
+            if result._specifier and not result.bound_item:
                 return False
-            if result.specifiers and not all(result.bound_item):
+            if result._specifiers and not all(result.bound_item):
                 return False
         return True
 
@@ -1527,7 +1594,7 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
             variable.connect_items(bound_items)
             variable.bound_item = BoundList(bound_items)
         else:
-            variable.bound_item = self.__resolve_object_specifier(variable.variable_specifier, variable.secondary_specifier, variable.property_name)
+            variable.bound_item = self.__resolve_object_specifier(variable._variable_specifier, variable._secondary_specifier, variable.property_name)
 
     def __unbind_variable(self, variable: ComputationVariable) -> None:
         self.__variable_changed_event_listeners[variable.uuid].close()
@@ -1601,7 +1668,7 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
                         with contextlib.closing(bound_item):
                             input_items.update(bound_item.base_objects)
             else:
-                bound_item = self.__resolve_object_specifier(variable.variable_specifier, variable.secondary_specifier, variable.property_name)
+                bound_item = self.__resolve_object_specifier(variable._variable_specifier, variable._secondary_specifier, variable.property_name)
                 if bound_item:
                     with contextlib.closing(bound_item):
                         input_items.update(bound_item.base_objects)
@@ -1610,13 +1677,13 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
     def get_preliminary_output_items(self) -> typing.Set:
         output_items = set()
         for result in self.results:
-            if result.specifier:
-                bound_item = self.__resolve_object_specifier(result.specifier)
+            if result._specifier:
+                bound_item = self.__resolve_object_specifier(result._specifier)
                 if bound_item:
                     with contextlib.closing(bound_item):
                         output_items.update(bound_item.base_objects)
-            elif result.specifiers is not None:
-                for specifier in result.specifiers:
+            elif result._specifiers is not None:
+                for specifier in result._specifiers:
                     bound_item = self.__resolve_object_specifier(specifier)
                     if bound_item:
                         with contextlib.closing(bound_item):
@@ -1651,19 +1718,19 @@ class Computation(Observable.Observable, Persistence.PersistentObject):
                 assert input_item.type is None
                 assert input_item.secondary_item is None
                 assert input_item.items is None
-                variable.specifier = DataStructure.get_object_specifier(input_item.item, allow_partial=False)
+                variable._specifier = DataStructure.get_object_specifier(input_item.item, allow_partial=False)
 
     def set_output_item(self, name:str, output_item: ComputationItem) -> None:
         for result in self.results:
             if result.name == name:
                 if output_item and output_item.items is not None:
-                    result.specifiers = [DataStructure.get_object_specifier(o.item, allow_partial=False) for o in output_item.items]
+                    result._specifiers = [DataStructure.get_object_specifier(o.item, allow_partial=False) for o in output_item.items]
                 else:
                     if output_item:
                         assert output_item.item
                         assert output_item.type is None
                         assert output_item.secondary_item is None
-                    result.specifier = DataStructure.get_object_specifier(output_item.item, allow_partial=False) if output_item else None
+                    result._specifier = DataStructure.get_object_specifier(output_item.item, allow_partial=False) if output_item else None
 
     def get_input(self, name: str):
         for variable in self.variables:
