@@ -1,4 +1,5 @@
 # standard libraries
+import contextlib
 import logging
 import math
 import unittest
@@ -19,6 +20,10 @@ from nion.ui import DrawingContext
 from nion.ui import TestUI
 
 
+def create_memory_profile_context() -> TestContext.MemoryProfileContext:
+    return TestContext.MemoryProfileContext()
+
+
 class TestLineGraphCanvasItem(unittest.TestCase):
 
     def setUp(self):
@@ -344,6 +349,48 @@ class TestLineGraphCanvasItem(unittest.TestCase):
             # ensure that the drawing commands are sufficiently populated to have drawn the graph
             self.assertGreater(len(drawing_context.commands), 100)
 
+    def test_line_plot_with_many_lines_displays_gracefully(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.ones((100,100), numpy.float))
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item.display_type = "line_plot"
+            display_panel = document_controller.selected_display_panel
+            display_panel.set_display_panel_display_item(display_item)
+            display_panel.display_canvas_item.layout_immediate((640, 480))
+
+    def test_line_plot_with_too_few_layers_displays_gracefully(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.ones((8,100), numpy.float))
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_item.display_type = "line_plot"
+            self.assertEqual(8, len(display_item.display_layers))
+            display_item.display_layers = display_item.display_layers[0:4]
+            self.assertEqual(4, len(display_item.display_layers))
+            display_panel = document_controller.selected_display_panel
+            display_panel.set_display_panel_display_item(display_item)
+            display_panel.display_canvas_item.layout_immediate((640, 480))
+
+    def test_line_plot_displays_gracefully_when_switching_layers(self):
+        with TestContext.create_memory_context() as test_context:
+            document_controller = test_context.create_document_controller()
+            document_model = document_controller.document_model
+            data_item = DataItem.DataItem(numpy.ones((8,100), numpy.float))
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            display_panel = document_controller.selected_display_panel
+            display_panel.set_display_panel_display_item(display_item)
+            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_item.display_type = "line_plot"
+            display_panel.display_canvas_item.layout_immediate((640, 480))
+            display_item.display_type = None
+            display_panel.display_canvas_item.layout_immediate((640, 480))
+
     def test_check_exponents(self):
         e = LineGraphCanvasItem.Exponenter()
         e.add_label("5e+05")
