@@ -871,11 +871,12 @@ class TestProcessingClass(unittest.TestCase):
             data_item2 = DataItem.DataItem(numpy.random.randn(8))
             document_model.append_data_item(data_item2)
             self.assertIsNone(document_controller._get_two_data_sources())
+            document_controller.selected_display_panel = None  # use data panel selection
             document_controller.select_data_items_in_data_panel([data_item1, data_item2])
             data_sources = document_controller._get_two_data_sources()
-            self.assertEqual(data_sources[0][0], document_model.get_display_item_for_data_item(data_item1))
+            self.assertIn(document_model.get_display_item_for_data_item(data_item1), (data_sources[0][0], data_sources[1][0]))
+            self.assertIn(document_model.get_display_item_for_data_item(data_item2), (data_sources[0][0], data_sources[1][0]))
             self.assertEqual(data_sources[0][1], None)
-            self.assertEqual(data_sources[1][0], document_model.get_display_item_for_data_item(data_item2))
             self.assertEqual(data_sources[1][1], None)
 
     def test_get_two_data_sources_handles_three_selected_data_items(self):
@@ -1085,6 +1086,18 @@ class TestProcessingClass(unittest.TestCase):
             document_model.get_processing_new("mapped_sum", display_item, display_item.data_item, crop_region)
             document_model.recompute_all()
 
+    def test_line_profile_on_sequence_works(self):
+        with TestContext.create_memory_context() as test_context:
+            document_model = test_context.create_document_model()
+            data_and_metadata = DataAndMetadata.new_data_and_metadata(numpy.zeros((20, 8, 6)), data_descriptor=DataAndMetadata.DataDescriptor(True, 0, 2))
+            data_item = DataItem.new_data_item(data_and_metadata)
+            document_model.append_data_item(data_item)
+            display_item = document_model.get_display_item_for_data_item(data_item)
+            line_profile_data_item = document_model.get_line_profile_new(display_item, data_item)
+            document_model.recompute_all()
+            self.assertIsNotNone(line_profile_data_item.xdata)
+            self.assertEqual((5, ), line_profile_data_item.xdata.data_shape)
+
 
 if __name__ == '__main__':
     logging.getLogger().setLevel(logging.DEBUG)
